# -*- encoding: utf-8 -*-
import collections
import copy
import math
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools.datastructuretools.TypedList import TypedList


class TimespanInventory(TypedList):
    r'''A timespan inventory.

    ..  container:: example

        **Example 1:**

        ::

            >>> timespan_inventory_1 = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10),
            ...     ])

        ::

            >>> print format(timespan_inventory_1)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(3, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(3, 1),
                        stop_offset=durationtools.Offset(6, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(6, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

    ..  container:: example

        **Example 2:**

        ::

            >>> timespan_inventory_2 = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(15, 20),
            ...     ])

        ::

            >>> print format(timespan_inventory_2)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(3, 1),
                        stop_offset=durationtools.Offset(6, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(15, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

    ..  container:: example

        **Example 3.** Empty timespan inventory:

        ::

            >>> timespan_inventory_3 = timespantools.TimespanInventory()

        ::

            >>> print format(timespan_inventory_3)
            timespantools.TimespanInventory(
                []
                )

    Operations on timespan currently work in place.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __and__(self, timespan):
        r'''Keep material that intersects `timespan`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 16),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 8),
            ...     ])

        ::

            >>> timespan = timespantools.Timespan(5, 10)
            >>> result = timespan_inventory & timespan

        ::

            >>> print format(timespan_inventory)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(8, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        Operates in place and returns timespan inventory.
        '''
        new_timespans = []
        for current_timespan in self[:]:
            result = current_timespan & timespan
            new_timespans.extend(result)
        self[:] = sorted(new_timespans)
        return self

    def __sub__(self, timespan):
        r'''Delete material that intersects `timespan`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 16),
            ...     timespantools.Timespan(5, 12),
            ...     timespantools.Timespan(-2, 8),
            ...     ])

        ::

            >>> timespan = timespantools.Timespan(5, 10)
            >>> result = timespan_inventory - timespan

        ::

            >>> print format(timespan_inventory)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(5, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(5, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(16, 1),
                        ),
                    ]
                )

        Operates in place and returns timespan inventory.
        '''
        new_timespans = []
        for current_timespan in self[:]:
            result = current_timespan - timespan
            new_timespans.extend(result)
        self[:] = sorted(new_timespans)
        return self

    ### PRIVATE METHODS ###

    def _get_offsets(self, expr):
        if hasattr(expr, 'start_offset') and hasattr(expr, 'stop_offset'):
            return expr.start_offset, expr.stop_offset
        elif hasattr(expr, 'timespan'):
            return expr.timespan.offsets
        else:
            raise TypeError(expr)

    def _get_timespan(self, expr):
        from abjad.tools import timespantools
        start_offset, stop_offset = self._get_offsets(expr)
        return timespantools.Timespan(start_offset, stop_offset)

    ### PUBLIC PROPERTIES ###

    @property
    def all_are_contiguous(self):
        r'''True when all timespans are time-contiguous:

        ::

            >>> timespan_inventory_1.all_are_contiguous
            True

        False when timespans not time-contiguous:

        ::

            >>> timespan_inventory_2.all_are_contiguous
            False

        True when empty:

        ::

            >>> timespan_inventory_3.all_are_contiguous
            True

        Returns boolean.
        '''
        if len(self) <= 1:
            return True
        timespans = sorted(self[:])
        last_stop_offset = timespans[0].stop_offset
        for timespan in timespans[1:]:
            if timespan.start_offset != last_stop_offset:
                return False
            last_stop_offset = timespan.stop_offset
        return True

    @property
    def all_are_nonoverlapping(self):
        r'''True when all timespans are non-overlapping:

        ::

            >>> timespan_inventory_1.all_are_nonoverlapping
            True

        False when timespans are overlapping:

        ::

            >>> timespan_inventory_2.all_are_nonoverlapping
            False

        True when empty:

        ::

            >>> timespan_inventory_3.all_are_nonoverlapping
            True

        Returns boolean.
        '''
        if len(self) <= 1:
            return True
        timespans = sorted(self[:])
        last_stop_offset = timespans[0].stop_offset
        for timespan in timespans[1:]:
            if timespan.start_offset < last_stop_offset:
                return False
            if last_stop_offset < timespan.stop_offset:
                last_stop_offset = timespan.stop_offset
        return True

    @property
    def all_are_well_formed(self):
        r'''True when all timespans are well-formed:

        ::

            >>> timespan_inventory_1.all_are_well_formed
            True

        ::

            >>> timespan_inventory_2.all_are_well_formed
            True

        Also true when empty:

        ::

            >>> timespan_inventory_3.all_are_well_formed
            True

        Otherwise false.

        Returns boolean.
        '''
        return all(self._get_timespan(expr).is_well_formed for expr in self)

    @property
    def axis(self):
        r'''Arithmetic mean of start- and stop-offsets.

            >>> timespan_inventory_1.axis
            Offset(5, 1)

        ::

            >>> timespan_inventory_2.axis
            Offset(10, 1)

        None when empty:

        ::

            >>> timespan_inventory_3.axis is None
            True

        Returns offset or none.
        '''
        if self:
            return (self.start_offset + self.stop_offset) / 2

    @property
    def duration(self):
        r'''Time from start offset to stop offset:

        ::

            >>> timespan_inventory_1.duration
            Duration(10, 1)

        ::

            >>> timespan_inventory_2.duration
            Duration(20, 1)

        Zero when empty:

        ::

            >>> timespan_inventory_3.duration
            Duration(0, 1)

        Returns duration.
        '''
        if self.stop_offset is not Infinity and \
            self.start_offset is not NegativeInfinity:
            return self.stop_offset - self.start_offset
        else:
            return durationtools.Duration(0)

    @property
    def is_sorted(self):
        r'''True when timespans are in time order:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10),
            ...     ])

        ::

            >>> timespan_inventory.is_sorted
            True

        Otherwise false:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(6, 10),
            ...     timespantools.Timespan(3, 6),
            ...     ])

        ::

            >>> timespan_inventory.is_sorted
            False

        Returns boolean.
        '''
        if len(self) < 2:
            return True
        for left_timespan, right_timespan in \
            sequencetools.iterate_sequence_pairwise_strict(self):
            if right_timespan.start_offset < left_timespan.start_offset:
                return False
            if left_timespan.start_offset == right_timespan.start_offset:
                if right_timespan.stop_offset < left_timespan.stop_offset:
                    return False
        return True

    @property
    def start_offset(self):
        r'''Earliest start offset of any timespan:

        ::

            >>> timespan_inventory_1.start_offset
            Offset(0, 1)

        ::

            >>> timespan_inventory_2.start_offset
            Offset(0, 1)

        Negative infinity when empty:

        ::

            >>> timespan_inventory_3.start_offset
            NegativeInfinity

        Returns offset or none.
        '''
        if self:
            return min([self._get_timespan(expr).start_offset
                for expr in self])
        else:
            return NegativeInfinity

    @property
    def stop_offset(self):
        r'''Latest stop offset of any timespan:

        ::

            >>> timespan_inventory_1.stop_offset
            Offset(10, 1)

        ::

            >>> timespan_inventory_2.stop_offset
            Offset(20, 1)

        Infinity when empty:

        ::

            >>> timespan_inventory_3.stop_offset
            Infinity

        Returns offset or none.
        '''
        if self:
            return max([self._get_timespan(expr).stop_offset for expr in self])
        else:
            return Infinity

    @property
    def timespan(self):
        r'''Timespan inventory timespan:

        ::

            >>> timespan_inventory_1.timespan
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))

        ::

            >>> timespan_inventory_2.timespan
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(20, 1))

        ::

            >>> timespan_inventory_3.timespan
            Timespan(start_offset=NegativeInfinity, stop_offset=Infinity)

        Returns timespan.
        '''
        from abjad.tools import timespantools
        return timespantools.Timespan(self.start_offset, self.stop_offset)

    ### PUBLIC METHODS ###

    def compute_logical_and(self):
        r'''Compute logical AND of timespans.

        ..  container:: example

            **Example 1:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_and()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(5, 12),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_and()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(5, 12),
                ...     timespantools.Timespan(-2, 8),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_and()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(8, 1),
                            ),
                        ]
                    )

        Same as setwise intersection.

        Operates in place and returns timespan inventory.
        '''
        if 1 < len(self):
            result = self[0]
            for timespan in self:
                if not timespan.intersects_timespan(result):
                    self[:] = []
                    return self
                else:
                    inventory = result & timespan
                    result = inventory[0]
            self[:] = [result]
        return self

    def compute_logical_or(self):
        r'''Compute logical OR of timespans.

        ..  container:: example

            **Example 1:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory()

            ::

                >>> result = timespan_inventory.compute_logical_or()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    []
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_or()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(5, 12),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_or()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(12, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 4:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(5, 12),
                ...     timespantools.Timespan(-2, 2),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_or()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-2, 1),
                            stop_offset=durationtools.Offset(12, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 5:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(-2, 2),
                ...     timespantools.Timespan(10, 20),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_or()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-2, 1),
                            stop_offset=durationtools.Offset(2, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(10, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        timespans = []
        if self:
            timespans = [self[0]]
            for timespan in self[1:]:
                if timespans[-1]._can_fuse(timespan):
                    inventory = timespans[-1] | timespan
                    timespans[-1:] = inventory[:]
                else:
                    timespans.append(timespan)
        self[:] = timespans
        return self

    def compute_logical_xor(self):
        r'''Compute logical XOR of timespans.

        ..  container:: example

            **Example 1:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory()

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    []
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(5, 12),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(5, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(10, 1),
                            stop_offset=durationtools.Offset(12, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 4:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(5, 12),
                ...     timespantools.Timespan(-2, 2),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-2, 1),
                            stop_offset=durationtools.Offset(0, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(2, 1),
                            stop_offset=durationtools.Offset(5, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(10, 1),
                            stop_offset=durationtools.Offset(12, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 5:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(-2, 2),
                ...     timespantools.Timespan(10, 20),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-2, 1),
                            stop_offset=durationtools.Offset(2, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(10, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 6:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(4, 8),
                ...     timespantools.Timespan(2, 6),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(2, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(8, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 7:**

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 10),
                ...     timespantools.Timespan(0, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.compute_logical_xor()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    []
                    )

        Operates in place and returns timespan inventory.
        '''
        all_fragments = []
        for i, timespan_1 in enumerate(self):
            timespan_1_fragments = [timespan_1]
            for j, timespan_2 in enumerate(self):
                if i == j:
                    continue
                revised_timespan_1_fragments = []
                for timespan_1_fragment in timespan_1_fragments:
                    if timespan_2.intersects_timespan(timespan_1_fragment):
                        result = timespan_1_fragment - timespan_2
                        revised_timespan_1_fragments.extend(result)
                    else:
                        revised_timespan_1_fragments.append(
                            timespan_1_fragment)
                timespan_1_fragments = revised_timespan_1_fragments
            all_fragments.extend(timespan_1_fragments)
        self[:] = all_fragments
        self.sort()
        return self

    def compute_overlap_factor(self, timespan=None):
        r'''Compute overlap factor of timespans:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 15),
            ...     timespantools.Timespan(20, 25),
            ...     timespantools.Timespan(20, 30),
            ...     ])

        ..  container:: example

            **Example 1.** Compute overlap factor across the entire inventory:

            ::

                >>> timespan_inventory.compute_overlap_factor()
                Multiplier(7, 6)

        ..  container:: example

            **Example 2a.** Compute overlap factor within a specific timespan:

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(-15, 0))
                Multiplier(0, 1)

        ..  container:: example

            **Example 2b:**

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(-10, 5))
                Multiplier(1, 3)

        ..  container:: example

            **Example 2c:**

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(-5, 10))
                Multiplier(1, 1)

        ..  container:: example

            **Example 2d:**

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(0, 15))
                Multiplier(4, 3)

        ..  container:: example

            **Example 2e:**

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(5, 20))
                Multiplier(1, 1)

        ..  container:: example

            **Example 2f:**

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(10, 25))
                Multiplier(1, 1)

        ..  container:: example

            **Example 2g:**

            ::

                >>> timespan_inventory.compute_overlap_factor(
                ...     timespan=timespantools.Timespan(15, 30))
                Multiplier(1, 1)

        Returns multiplier.
        '''
        from abjad.tools import timespantools
        if timespan is None:
            timespan = self.timespan
        time_relation = timespantools.timespan_2_intersects_timespan_1(
            timespan_1=timespan)
        timespan_inventory = self.get_timespans_that_satisfy_time_relation(
            time_relation)
        total_overlap = durationtools.Duration(sum(
            x.get_overlap_with_timespan(timespan) for x in timespan_inventory))
        overlap_factor = total_overlap / timespan.duration
        return overlap_factor

    def compute_overlap_factor_mapping(self):
        r'''Compute overlap factor for each consecutive offset pair in
        timespans:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 10),
            ...     timespantools.Timespan(5, 15),
            ...     timespantools.Timespan(20, 25),
            ...     timespantools.Timespan(20, 30),
            ...     ])

        ::

            >>> mapping = timespan_inventory.compute_overlap_factor_mapping()
            >>> for timespan, overlap_factor in mapping.iteritems():
            ...     timespan.start_offset, timespan.stop_offset, overlap_factor
            ...
            (Offset(0, 1), Offset(5, 1), Multiplier(1, 1))
            (Offset(5, 1), Offset(10, 1), Multiplier(2, 1))
            (Offset(10, 1), Offset(15, 1), Multiplier(1, 1))
            (Offset(15, 1), Offset(20, 1), Multiplier(0, 1))
            (Offset(20, 1), Offset(25, 1), Multiplier(2, 1))
            (Offset(25, 1), Offset(30, 1), Multiplier(1, 1))

        Returns mapping.
        '''
        from abjad.tools import timespantools
        mapping = collections.OrderedDict()
        for start_offset, stop_offset in \
            sequencetools.iterate_sequence_pairwise_strict(sorted(
                self.count_offsets())):
            timespan = timespantools.Timespan(start_offset, stop_offset)
            overlap_factor = self.compute_overlap_factor(timespan=timespan)
            mapping[timespan] = overlap_factor
        return mapping

    def count_offsets(self):
        r'''Count offsets in inventory:

        ..  container:: example

            **Example 1:**

            ::

                >>> print format(timespan_inventory_1)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

            ::

                >>> for offset, count in sorted(
                ...     timespan_inventory_1.count_offsets().iteritems()):
                ...     offset, count
                ...
                (Offset(0, 1), 1)
                (Offset(3, 1), 2)
                (Offset(6, 1), 2)
                (Offset(10, 1), 1)

        ..  container:: example

            **Example 2:**

            ::

                >>> print format(timespan_inventory_2)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

            ::

                >>> for offset, count in sorted(
                ...     timespan_inventory_2.count_offsets().iteritems()):
                ...     offset, count
                ...
                (Offset(0, 1), 1)
                (Offset(3, 1), 1)
                (Offset(6, 1), 1)
                (Offset(10, 1), 1)
                (Offset(15, 1), 1)
                (Offset(20, 1), 1)

        ..  container:: example

            **Example 3:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(0, 6),
                ...     timespantools.Timespan(0, 9),
                ...     ])
                >>> for offset, count in sorted(
                ...     timespan_inventory.count_offsets().iteritems()):
                ...     offset, count
                ...
                (Offset(0, 1), 3)
                (Offset(3, 1), 1)
                (Offset(6, 1), 1)
                (Offset(9, 1), 1)

        Returns counter.
        '''
        counter = collections.Counter()
        for timespan in self:
            counter[timespan.start_offset] += 1
            counter[timespan.stop_offset] += 1
        return counter

    def explode(self, inventory_count=None):
        r'''Explode timespans into inventories, avoiding overlap, and
        distributing density as evenly as possible.

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(5, 13),
            ...     timespantools.Timespan(6, 10),
            ...     timespantools.Timespan(8, 9),
            ...     timespantools.Timespan(15, 23),
            ...     timespantools.Timespan(16, 21),
            ...     timespantools.Timespan(17, 19),
            ...     timespantools.Timespan(19, 20),
            ...     timespantools.Timespan(25, 30),
            ...     timespantools.Timespan(26, 29),
            ...     timespantools.Timespan(32, 34),
            ...     timespantools.Timespan(34, 37),
            ...     ])

        ..  container:: example

            **Example 1.** Explode timespans into the optimal number of
            non-overlapping inventories:

            ::

                >>> for exploded_inventory in timespan_inventory.explode():
                ...     print format(exploded_inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(13, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(17, 1),
                            stop_offset=durationtools.Offset(19, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(19, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(34, 1),
                            stop_offset=durationtools.Offset(37, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(16, 1),
                            stop_offset=durationtools.Offset(21, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(25, 1),
                            stop_offset=durationtools.Offset(30, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(8, 1),
                            stop_offset=durationtools.Offset(9, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(23, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(26, 1),
                            stop_offset=durationtools.Offset(29, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(32, 1),
                            stop_offset=durationtools.Offset(34, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2.** Explode timespans into a less-than-optimal number of
            overlapping inventories:

            ::

                >>> for exploded_inventory in timespan_inventory.explode(
                ...     inventory_count=2):
                ...     print format(exploded_inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(13, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(23, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(25, 1),
                            stop_offset=durationtools.Offset(30, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(34, 1),
                            stop_offset=durationtools.Offset(37, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(8, 1),
                            stop_offset=durationtools.Offset(9, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(16, 1),
                            stop_offset=durationtools.Offset(21, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(17, 1),
                            stop_offset=durationtools.Offset(19, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(19, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(26, 1),
                            stop_offset=durationtools.Offset(29, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(32, 1),
                            stop_offset=durationtools.Offset(34, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3.** Explode timespans into a greater-than-optimal number
            of non-overlapping inventories:

            ::

                >>> for exploded_inventory in timespan_inventory.explode(
                ...     inventory_count=6):
                ...     print format(exploded_inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(16, 1),
                            stop_offset=durationtools.Offset(21, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(34, 1),
                            stop_offset=durationtools.Offset(37, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(23, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(8, 1),
                            stop_offset=durationtools.Offset(9, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(17, 1),
                            stop_offset=durationtools.Offset(19, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(19, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(26, 1),
                            stop_offset=durationtools.Offset(29, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(32, 1),
                            stop_offset=durationtools.Offset(34, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(13, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(25, 1),
                            stop_offset=durationtools.Offset(30, 1),
                            ),
                        ]
                    )

        Returns inventories.
        '''
        assert isinstance(inventory_count, (type(None), int))
        if isinstance(inventory_count, int):
            assert 0 < inventory_count
        bounding_timespan = self.timespan
        global_overlap_factors = []
        empty_inventory_pairs = []
        result_inventories = []
        if inventory_count is not None:
            for i in range(inventory_count):
                global_overlap_factors.append(0)
                result_inventory = type(self)([])
                empty_inventory_pairs.append((i, result_inventory))
                result_inventories.append(result_inventory)
        for current_timespan in self:
            current_overlap_factor = \
                current_timespan.duration / bounding_timespan.duration
            if empty_inventory_pairs:
                i, empty_inventory = empty_inventory_pairs.pop()
                empty_inventory.append(current_timespan)
                global_overlap_factors[i] = current_overlap_factor
                continue
            nonoverlapping_inventories = []
            overlapping_inventories = []
            for i, result_inventory in enumerate(result_inventories):
                local_overlap_factor = result_inventory.compute_overlap_factor(
                    current_timespan)
                global_overlap_factor = global_overlap_factors[i]
                if not local_overlap_factor:
                    nonoverlapping_inventories.append(
                        (i, global_overlap_factor))
                else:
                    overlapping_inventories.append(
                        (i, local_overlap_factor, global_overlap_factor))
            nonoverlapping_inventories.sort(key=lambda x: x[1])
            overlapping_inventories.sort(key=lambda x: (x[1], x[2]))
            if not nonoverlapping_inventories and inventory_count is None:
                result_inventory = type(self)([current_timespan])
                global_overlap_factors.append(current_overlap_factor)
                result_inventories.append(result_inventory)
                continue
            if nonoverlapping_inventories:
                i = nonoverlapping_inventories[0][0]
            else:
                i = overlapping_inventories[0][0]
            result_inventory = result_inventories[i]
            result_inventory.append(current_timespan)
            global_overlap_factors[i] += current_overlap_factor
        return tuple(result_inventories)

    def get_timespan_that_satisfies_time_relation(self, time_relation):
        r'''Get timespan that satisifies `time_relation`:

        ::

            >>> timespan_1 = timespantools.Timespan(2, 5)
            >>> time_relation = \
            ...     timespantools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> timespan_inventory_1.get_timespan_that_satisfies_time_relation(
            ...     time_relation)
            Timespan(start_offset=Offset(3, 1), stop_offset=Offset(6, 1))

        Returns timespan when timespan inventory contains exactly one
        timespan that satisfies `time_relation`.

        Raises exception when timespan inventory contains no timespan
        that satisfies `time_relation`.

        Raises exception when timespan inventory contains more than one
        timespan that satisfies `time_relation`.
        '''
        timespans = self.get_timespans_that_satisfy_time_relation(
            time_relation)
        if len(timespans) == 1:
            return timespans[0]
        elif 1 < len(timespans):
            message = 'extra timespan.'
            raise Exception(message)
        else:
            message = 'missing timespan.'
            raise Exception(message)

    def get_timespans_that_satisfy_time_relation(self, time_relation):
        r'''Get timespans that satisfy `time_relation`:

        ::

            >>> timespan_1 = timespantools.Timespan(2, 8)
            >>> time_relation = \
            ...     timespantools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> result = \
            ...     timespan_inventory_1.get_timespans_that_satisfy_time_relation(
            ...     time_relation)

        ::

            >>> print format(result)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(3, 1),
                        stop_offset=durationtools.Offset(6, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(6, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        Returns new timespan inventory.
        '''
        from abjad.tools import timespantools
        result = []
        for timespan in self:
            if isinstance(
                time_relation,
                timespantools.TimespanTimespanTimeRelation):
                if time_relation(timespan_2=timespan):
                    result.append(timespan)
            elif isinstance(
                time_relation,
                timespantools.OffsetTimespanTimeRelation):
                if time_relation(timespan=timespan):
                    result.append(timespan)
            else:
                raise ValueError
        return type(self)(result)

    def has_timespan_that_satisfies_time_relation(self, time_relation):
        r'''True when timespan inventory has timespan that
        satisfies `time_relation`:

        ::

            >>> timespan_1 = timespantools.Timespan(2, 8)
            >>> time_relation = \
            ...     timespantools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> timespan_inventory_1.has_timespan_that_satisfies_time_relation(
            ...     time_relation)
            True

        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(10, 20)
            >>> time_relation = \
            ...     timespantools.timespan_2_starts_during_timespan_1(
            ...     timespan_1=timespan_1)

        ::

            >>> timespan_inventory_1.has_timespan_that_satisfies_time_relation(
            ...     time_relation)
            False

        Returns boolean.
        '''
        return bool(
            self.get_timespans_that_satisfy_time_relation(time_relation))

    def partition(self, include_tangent_timespans=False):
        r'''Partition timespans into inventories:

        ..  container:: example

            **Example 1:**

            ::

                >>> print format(timespan_inventory_1)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

            ::

                >>> for inventory in timespan_inventory_1.partition():
                ...     print format(inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> print format(timespan_inventory_2)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

            ::

                >>> for inventory in timespan_inventory_2.partition():
                ...     print format(inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3.** Treat tangent timespans as part of the same group with
            include_tangent_timespans:

            ::

                >>> for inventory in timespan_inventory_1.partition(
                ...     include_tangent_timespans=True):
                ...     print format(inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        Returns 0 or more inventories.
        '''
        if not self:
            return []
        inventories = []
        timespans = sorted(self[:])
        current_inventory = type(self)([timespans[0]])
        latest_stop_offset = current_inventory[0].stop_offset
        for current_timespan in timespans[1:]:
            if current_timespan.start_offset < latest_stop_offset:
                current_inventory.append(current_timespan)
            elif include_tangent_timespans and \
                current_timespan.start_offset == latest_stop_offset:
                current_inventory.append(current_timespan)
            else:
                inventories.append(current_inventory)
                current_inventory = type(self)([current_timespan])
            if latest_stop_offset < current_timespan.stop_offset:
                latest_stop_offset = current_timespan.stop_offset
        if current_inventory:
            inventories.append(current_inventory)
        return tuple(inventories)

    def reflect(self, axis=None):
        r'''Reflect timespans.

        ..  container:: example

            **Example 1.** Reflect timespans about timespan inventory axis:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.reflect()

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(4, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(4, 1),
                            stop_offset=durationtools.Offset(7, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(7, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2.** Reflect timespans about arbitrary axis:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.reflect(axis=Offset(15))

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(20, 1),
                            stop_offset=durationtools.Offset(24, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(24, 1),
                            stop_offset=durationtools.Offset(27, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(27, 1),
                            stop_offset=durationtools.Offset(30, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        if axis is None:
            axis = self.axis
        timespans = []
        for timespan in self:
            timespan = timespan.reflect(axis=axis)
            timespans.append(timespan)
        timespans.reverse()
        self[:] = timespans
        return self

    def remove_degenerate_timespans(self):
        r'''Remove degenerate timespans:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(5, 5),
            ...     timespantools.Timespan(5, 10),
            ...     timespantools.Timespan(5, 25),
            ...     ])

        ::

            >>> result = timespan_inventory.remove_degenerate_timespans()

        ::

            >>> print format(timespan_inventory)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(25, 1),
                        ),
                    ]
                )

        Operates in place and returns timespan inventory.
        '''
        timespans = [x for x in self if x.is_well_formed]
        self[:] = timespans
        return self

    def repeat_to_stop_offset(self, stop_offset):
        r'''Repeat timespans to `stop_offset`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10),
            ...     ])

        ::

            >>> result = timespan_inventory.repeat_to_stop_offset(15)

        ::

            >>> print format(timespan_inventory)
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(3, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(3, 1),
                        stop_offset=durationtools.Offset(6, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(6, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(13, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(13, 1),
                        stop_offset=durationtools.Offset(15, 1),
                        ),
                    ]
                )

        Operates in place and returns timespan inventory.
        '''
        assert self.is_sorted
        stop_offset = durationtools.Offset(stop_offset)
        assert self.stop_offset <= stop_offset
        current_timespan_index = 0
        if self:
            while self.stop_offset < stop_offset:
                current_timespan = self[current_timespan_index]
                translation = self.stop_offset - current_timespan.start_offset
                new_timespan = current_timespan.translate(translation)
                self.append(new_timespan)
                current_timespan_index += 1
            if stop_offset < self.stop_offset:
                self[-1] = self[-1].set_offsets(stop_offset=stop_offset)
        return self

    def rotate(self, count):
        r'''Rotate by `count` contiguous timespans.

        ..  container:: example

            **Example 1.** Rotate by one timespan to the left:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 4),
                ...     timespantools.Timespan(4, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.rotate(-1)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(1, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(1, 1),
                            stop_offset=durationtools.Offset(7, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(7, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2.** Rotate by one timespan to the right:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 4),
                ...     timespantools.Timespan(4, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.rotate(1)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(9, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(9, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        assert isinstance(count, int)
        assert self.all_are_contiguous
        elements_to_move = count % len(self)
        if elements_to_move == 0:
            return
        left_timespans = self[:-elements_to_move]
        right_timespans = self[-elements_to_move:]
        split_offset = right_timespans[0].start_offset
        translation_to_left = split_offset - self.start_offset
        translation_to_left *= -1
        translation_to_right = self.stop_offset - split_offset
        translated_right_timespans = []
        for right_timespan in right_timespans:
            translated_right_timespan = right_timespan.translate_offsets(
                translation_to_left, translation_to_left)
            translated_right_timespans.append(translated_right_timespan)
        translated_left_timespans = []
        for left_timespan in left_timespans:
            translated_left_timespan = left_timespan.translate_offsets(
                translation_to_right, translation_to_right)
            translated_left_timespans.append(translated_left_timespan)
        new_timespans = translated_right_timespans + translated_left_timespans
        self[:] = new_timespans
        return self

    def round_offsets(self, multiplier, anchor=Left, must_be_well_formed=True):
        '''Round offsets of timespans in inventory to multiples of
        `multiplier`:

        ..  container:: example

            **Example 1:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 2),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> rounded_inventory = timespan_inventory.round_offsets(3)
                >>> print format(rounded_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(9, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 2),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> rounded_inventory = timespan_inventory.round_offsets(5)
                >>> print format(rounded_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(5, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 2),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> rounded_inventory = timespan_inventory.round_offsets(
                ...     5,
                ...     anchor=Right,
                ...     )
                >>> print format(rounded_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-5, 1),
                            stop_offset=durationtools.Offset(0, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(5, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 4:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 2),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> rounded_inventory = timespan_inventory.round_offsets(
                ...     5,
                ...     anchor=Right,
                ...     must_be_well_formed=False,
                ...     )
                >>> print format(rounded_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(0, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(5, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(5, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        timespans = []
        for timespan in self:
            timespan = timespan.round_offsets(
                multiplier,
                anchor=anchor,
                must_be_well_formed=must_be_well_formed,
                )
            timespans.append(timespan)
        self[:] = timespans
        return self

    def scale(self, multiplier, anchor=Left):
        r'''Scale timespan by `multiplier` relative to `anchor`.

        ..  container:: example

            **Example 1.** Scale timespans relative to timespan inventory start offset:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.scale(2)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(9, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(14, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2.** Scale timespans relative to timespan inventory stop offset:

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.scale(2, anchor=Right)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-3, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(2, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        timespans = []
        for timespan in self:
            timespan = timespan.scale(multiplier, anchor=anchor)
            timespans.append(timespan)
        self[:] = timespans
        return self

    def split_at_offset(self, offset):
        '''Split timespans at `offset`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(6, 10),
            ...     ])

        ..  container:: example

            **Example 1:**

            ::

                >>> left, right = timespan_inventory.split_at_offset(4)

            ::

                >>> print format(left)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(4, 1),
                            ),
                        ]
                    )

            ::

                >>> print format(right)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(4, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> left, right = timespan_inventory.split_at_offset(6)

            ::

                >>> print format(left)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        ]
                    )

            ::

                >>> print format(right)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 3:**

            ::

                >>> left, right = timespan_inventory.split_at_offset(-1)

            ::

                >>> print format(left)
                timespantools.TimespanInventory(
                    []
                    )

            ::

                >>> print format(right)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )

        Returns inventories.
        '''
        from abjad.tools import timespantools
        offset = durationtools.Offset(offset)
        during_time_relation = timespantools.OffsetTimespanTimeRelation(
            timespantools.CompoundInequality([
                'timespan.start < offset',
                'offset < timespan.stop'],
                logical_operator='and',
                ),
            offset=offset,
            )
        during_timespans = self.get_timespans_that_satisfy_time_relation(
            during_time_relation)
        before_time_relation = timespantools.OffsetTimespanTimeRelation(
            timespantools.CompoundInequality([
                'timespan.stop <= offset'],
                ),
            offset=offset,
            )
        before_timespans = self.get_timespans_that_satisfy_time_relation(
            before_time_relation)
        after_time_relation = timespantools.OffsetTimespanTimeRelation(
            timespantools.CompoundInequality([
                'offset <= timespan.start'],
                ),
            offset=offset,
            )
        after_timespans = self.get_timespans_that_satisfy_time_relation(
            after_time_relation)
        before_inventory = type(self)(before_timespans)
        after_inventory = type(self)(after_timespans)
        for timespan in during_timespans:
            before_timespan, after_timespan = timespan.split_at_offset(offset)
            before_inventory.append(before_timespan)
            after_inventory.append(after_timespan)
        before_inventory.sort()
        after_inventory.sort()
        return before_inventory, after_inventory

    def split_at_offsets(self, offsets):
        '''Split timespans at `offsets`:

        ::

            >>> timespan_inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 3),
            ...     timespantools.Timespan(3, 6),
            ...     timespantools.Timespan(4, 10),
            ...     timespantools.Timespan(15, 20),
            ...     ])

        ::

            >>> offsets = [-1, 3, 6, 12, 13]

        ..  container:: example

            **Example 1:**

            ::

                >>> for inventory in timespan_inventory.split_at_offsets(offsets):
                ...     print format(inventory)
                ...
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(3, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(4, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(10, 1),
                            ),
                        ]
                    )
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(15, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2:**

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([])
                >>> timespan_inventory.split_at_offsets(offsets)
                [TimespanInventory([])]

        Returns 1 or more inventories.
        '''
        inventories = [self]
        if not self:
            return inventories
        offsets = sorted(set(durationtools.Offset(x) for x in offsets))
        offsets = [x for x in offsets
            if self.start_offset < x < self.stop_offset]
        for offset in offsets:
            shards = [x for x in inventories[-1].split_at_offset(offset)
                if x]
            if shards:
                inventories[-1:] = shards
        return inventories

    def stretch(self, multiplier, anchor=None):
        r'''Stretch timespans by `multiplier` relative to `anchor`.

        ..  container:: example

            **Example 1:** Stretch timespans relative to timespan inventory
            start offset:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.stretch(2)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(6, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(12, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(12, 1),
                            stop_offset=durationtools.Offset(20, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2:** Stretch timespans relative to arbitrary anchor:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])


            ::

                >>> result = timespan_inventory.stretch(2, anchor=Offset(8))

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-8, 1),
                            stop_offset=durationtools.Offset(-2, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(-2, 1),
                            stop_offset=durationtools.Offset(4, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(4, 1),
                            stop_offset=durationtools.Offset(12, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        timespans = []
        if anchor is None:
            anchor = self.start_offset
        for timespan in self:
            timespan = timespan.stretch(multiplier, anchor)
            timespans.append(timespan)
        self[:] = timespans
        return self

    def translate(self, translation=None):
        r'''Translate timespans by `translation`.

        ..  container:: example

            **Example 1.** Translate timespan by offset ``50``:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.translate(50)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(50, 1),
                            stop_offset=durationtools.Offset(53, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(53, 1),
                            stop_offset=durationtools.Offset(56, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(56, 1),
                            stop_offset=durationtools.Offset(60, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        return self.translate_offsets(translation, translation)

    def translate_offsets(
        self,
        start_offset_translation=None,
        stop_offset_translation=None,
        ):
        r'''Translate timespans by `start_offset_translation`
        and `stop_offset_translation`.

        ..  container:: example

            **Example 1.** Translate timespan start- and stop-offsets equally:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.translate_offsets(50, 50)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(50, 1),
                            stop_offset=durationtools.Offset(53, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(53, 1),
                            stop_offset=durationtools.Offset(56, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(56, 1),
                            stop_offset=durationtools.Offset(60, 1),
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2.** Translate timespan stop-offsets only:

            ::

                >>> timespan_inventory = timespantools.TimespanInventory([
                ...     timespantools.Timespan(0, 3),
                ...     timespantools.Timespan(3, 6),
                ...     timespantools.Timespan(6, 10),
                ...     ])

            ::

                >>> result = timespan_inventory.translate_offsets(
                ...     stop_offset_translation=20)

            ::

                >>> print format(timespan_inventory)
                timespantools.TimespanInventory(
                    [
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(0, 1),
                            stop_offset=durationtools.Offset(23, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(3, 1),
                            stop_offset=durationtools.Offset(26, 1),
                            ),
                        timespantools.Timespan(
                            start_offset=durationtools.Offset(6, 1),
                            stop_offset=durationtools.Offset(30, 1),
                            ),
                        ]
                    )

        Operates in place and returns timespan inventory.
        '''
        timespans = []
        for timespan in self:
            timespan = timespan.translate_offsets(
                start_offset_translation,
                stop_offset_translation,
                )
            timespans.append(timespan)
        self[:] = timespans
        return self



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def is_integer_equivalent_n_tuple(expr, n):
    r'''True when `expr` is a tuple of `n` integer-equivalent expressions:

    ::

        >>> sequencetools.is_integer_equivalent_n_tuple((2.0, '3', Fraction(4, 1)), 3)
        True

    Otherwise false:

    ::

        >>> sequencetools.is_integer_equivalent_n_tuple((2.5, '3', Fraction(4, 1)), 3)
        False

    Returns boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == n and \
        all(mathtools.is_integer_equivalent_expr(x) for x in expr)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.demos.part.PartCantusScoreTemplate import PartCantusScoreTemplate
from abjad.demos.part.add_bell_music_to_score import add_bell_music_to_score
from abjad.demos.part.add_string_music_to_score import add_string_music_to_score
from abjad.demos.part.apply_bowing_marks import apply_bowing_marks
from abjad.demos.part.apply_dynamics import apply_dynamics
from abjad.demos.part.apply_expressive_marks import apply_expressive_marks
from abjad.demos.part.apply_final_bar_lines import apply_final_bar_lines
from abjad.demos.part.apply_page_breaks import apply_page_breaks
from abjad.demos.part.apply_rehearsal_marks import apply_rehearsal_marks
from abjad.demos.part.configure_lilypond_file import configure_lilypond_file
from abjad.demos.part.configure_score import configure_score


def make_part_lilypond_file():
    r'''Makes Pärt LilyPond file.
    '''

    score_template = PartCantusScoreTemplate()
    score = score_template()

    add_bell_music_to_score(score)
    add_string_music_to_score(score)

    apply_bowing_marks(score)
    apply_dynamics(score)
    apply_expressive_marks(score)
    apply_page_breaks(score)
    apply_rehearsal_marks(score)
    apply_final_bar_lines(score)

    configure_score(score)
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    configure_lilypond_file(lilypond_file)

    return lilypond_file



# -*- encoding: utf-8 -*-
from abjad.tools.stringtools.is_snake_case_string \
	import underscore_delimited_lowercase_regex_body
import re


underscore_delimited_lowercase_package_regex_body = """
    (%s\.)*
    %s
    """ % (underscore_delimited_lowercase_regex_body, underscore_delimited_lowercase_regex_body)

underscore_delimited_lowercase_package_regex = re.compile('^%s$' %
    underscore_delimited_lowercase_package_regex_body, re.VERBOSE)

def is_snake_case_package_name(expr):
    r'''True when `expr` is a string and is underscore-delimited lowercase package name:

    ::

        >>> stringtools.is_snake_case_package_name('foo.bar.blah_package')
        True

    Otherwise false:

    ::

        >>> stringtools.is_snake_case_package_name('foo.bar.BlahPackage')
        False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(underscore_delimited_lowercase_package_regex.match(expr))



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def pair_duration_sequence_elements_with_input_pair_values(duration_sequence, input_pairs):
    r'''Pair `duration_sequence` elements with the values of `input_pairs`:

    ::

        >>> duration_sequence = [10, 10, 10, 10]
        >>> input_pairs = [('red', 1), ('orange', 18), ('yellow', 200)]

    ::

        >>> sequencetools.pair_duration_sequence_elements_with_input_pair_values(
        ... duration_sequence, input_pairs)
        [(10, 'red'), (10, 'orange'), (10, 'yellow'), (10, 'yellow')]

    Returns a list of ``(element, value)`` output pairs.

    The `input_pairs` argument must be a list of ``(value, duration)`` pairs.

    The basic idea behind the function is model which input pair
    value is in effect at the start of each element in `duration_sequence`.
    '''
    from abjad.tools import sequencetools

    assert sequencetools.all_are_numbers(duration_sequence)
    assert sequencetools.all_are_pairs(input_pairs)

    output_pairs = []
    current_element_start = 0
    current_input_pair_index = 0
    current_input_pair = input_pairs[current_input_pair_index]
    current_input_pair_value = current_input_pair[0]
    current_input_pair_duration = current_input_pair[-1]
    current_input_pair_start = 0
    current_input_pair_stop = current_input_pair_start + current_input_pair_duration

    for element in duration_sequence:
        while current_input_pair_stop <= current_element_start:
            current_input_pair_index += 1
            current_input_pair = input_pairs[current_input_pair_index]
            current_input_pair_value = current_input_pair[0]
            current_input_pair_duration = current_input_pair[-1]
            current_input_pair_start = current_input_pair_stop
            current_input_pair_stop += current_input_pair_duration
        output_pair = (element, current_input_pair_value)
        output_pairs.append(output_pair)
        current_element_start += element

    return output_pairs



# -*- encoding: utf-8 -*-


def pluralize_string(string):
    r'''Pluralize English `string`. Change terminal `-y` to `-ies`:

    ::

        >>> stringtools.pluralize_string('catenary')
        'catenaries'

    Add `-es` to terminal `-s`, `-sh`, `-x` and `-z`:

    ::

        >>> stringtools.pluralize_string('brush')
        'brushes'

    Add `-s` to all other strings:

    ::

        >>> stringtools.pluralize_string('shape')
        'shapes'

    Returns string.
    '''
    if string.endswith('y'):
        return string[:-1] + 'ies'
    elif string.endswith(('s', 'sh', 'x', 'z')):
        return string + 'es'
    else:
        return string + 's'



# -*- encoding: utf-8 -*-
from abjad.tools import documentationtools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import spannertools
from abjad.tools import scoretools
from abjad.tools.datastructuretools.TreeContainer import TreeContainer
from abjad.tools.rhythmtreetools.RhythmTreeNode import RhythmTreeNode


class RhythmTreeContainer(RhythmTreeNode, TreeContainer):
    r'''A container node in a rhythm tree structure:

    ::

        >>> container = rhythmtreetools.RhythmTreeContainer(
        ...     preprolated_duration=1, children=[])
        >>> container
        RhythmTreeContainer(
            preprolated_duration=Duration(1, 1)
            )

    Similar to Abjad containers, `RhythmTreeContainer` supports a list 
    interface, and can be appended, extended, indexed and so forth 
    by other `RhythmTreeNode` subclasses:

    ::

        >>> leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)
        >>> leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
        >>> container.extend([leaf_a, leaf_b])
        >>> container
        RhythmTreeContainer(
            children=(
                RhythmTreeLeaf(
                    preprolated_duration=Duration(1, 1),
                    is_pitched=True
                    ),
                RhythmTreeLeaf(
                    preprolated_duration=Duration(2, 1),
                    is_pitched=True
                    ),
                ),
            preprolated_duration=Duration(1, 1)
            )

    ::

        >>> another_container = rhythmtreetools.RhythmTreeContainer(
        ...     preprolated_duration=2)
        >>> another_container.append(
        ...     rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3))
        >>> another_container.append(container[1])
        >>> container.append(another_container)
        >>> container
        RhythmTreeContainer(
            children=(
                RhythmTreeLeaf(
                    preprolated_duration=Duration(1, 1),
                    is_pitched=True
                    ),
                RhythmTreeContainer(
                    children=(
                        RhythmTreeLeaf(
                            preprolated_duration=Duration(3, 1),
                            is_pitched=True
                            ),
                        RhythmTreeLeaf(
                            preprolated_duration=Duration(2, 1),
                            is_pitched=True
                            ),
                        ),
                    preprolated_duration=Duration(2, 1)
                    ),
                ),
            preprolated_duration=Duration(1, 1)
            )

    Call `RhythmTreeContainer` with a preprolated_duration to generate 
    a tuplet structure:

    ::

        >>> container((1, 4))
        [FixedDurationTuplet(Duration(1, 4), "c'8 {@ 5:4 c'8., c'8 @}")]

    ..  doctest::

        >>> print format(_[0])
        \times 2/3 {
            c'8
            \times 4/5 {
                c'8.
                c'8
            }
        }

    Returns `RhythmTreeContainer` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, children=None, preprolated_duration=1, name=None):
        RhythmTreeNode.__init__(
            self,
            preprolated_duration=preprolated_duration,
            name=name,
            )
        self._children = []
        if isinstance(children, type(None)):
            pass
        elif isinstance(children, (list, str, tuple)):
            self.extend(children)
        else:
            message = 'can not instantiate {} with {!r}.'
            raise ValueError(message.format(type(self), children))

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Concatenate containers self and expr. The operation c = a + b
        returns a new RhythmTreeContainer c with the content of both a and b,
        and a preprolated_duration equal to the sum of the durations 
        of a and b. The operation is non-commutative: the content of the 
        first operand will be placed before the content of the second operand:

        ::

            >>> a = rhythmtreetools.RhythmTreeParser()('(1 (1 1 1))')[0]
            >>> b = rhythmtreetools.RhythmTreeParser()('(2 (3 4))')[0]

        ::

            >>> c = a + b

        ::

            >>> c.preprolated_duration
            Duration(3, 1)

        ::

            >>> c
            RhythmTreeContainer(
                children=(
                    RhythmTreeLeaf(
                        preprolated_duration=Duration(1, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        preprolated_duration=Duration(1, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        preprolated_duration=Duration(1, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        preprolated_duration=Duration(3, 1),
                        is_pitched=True
                        ),
                    RhythmTreeLeaf(
                        preprolated_duration=Duration(4, 1),
                        is_pitched=True
                        ),
                    ),
                preprolated_duration=Duration(3, 1)
                )

        Returns new RhythmTreeContainer.
        '''
        from abjad.tools.rhythmtreetools.RhythmTreeParser \
            import RhythmTreeParser
        if isinstance(expr, str):
            expr = RhythmTreeParser()(expr)
            assert 1 == len(expr) and isinstance(expr[0], type(self))
            expr = expr[0]
        container = type(self)(
            preprolated_duration=self.preprolated_duration + 
            expr.preprolated_duration)
        container.extend(self[:])
        container.extend(expr[:])
        return container

    def __call__(self, pulse_duration):
        r'''Generate Abjad score components:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree((1, 4))
            [FixedDurationTuplet(Duration(1, 4), "c'16 {@ 3:2 c'16, c'16, c'16 @} c'8")]

        Returns sequence of components.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        assert 0 < pulse_duration
        def recurse(node, tuplet_duration):
            basic_prolated_duration = tuplet_duration / node._contents_duration
            basic_written_duration = \
                basic_prolated_duration.equal_or_greater_power_of_two
            tuplet = scoretools.FixedDurationTuplet(tuplet_duration, [])
            for child in node.children:
                if isinstance(child, type(self)):
                    tuplet.extend(recurse(
                        child,
                        child.preprolated_duration * basic_written_duration))
                else:
                    leaves = child(basic_written_duration)
                    tuplet.extend(leaves)
                    if 1 < len(leaves):
                        tie = spannertools.Tie()
                        attach(tie, leaves)
            if tuplet.multiplier == 1:
                return tuplet[:]
            return [tuplet]
        result = recurse(self, pulse_duration * self.preprolated_duration)
        for component in result[:]:
            if isinstance(component, scoretools.Tuplet):
                if component.is_trivial:
                    component._extract()
        return result

    def __eq__(self, expr):
        r'''True if type, preprolated_duration and children are equivalent.
        Otherwise False.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self.preprolated_duration == expr.preprolated_duration:
                if self.children == expr.children:
                    return True
        return False

    def __setitem__(self, i, expr):
        r'''Set `expr` in self at nonnegative integer index `i`, 
        or set `expr` in self at slice i.
        Replace contents of `self[i]` with `expr`.
        Attach parentage to contents of `expr`, 
        and detach parentage of any replaced nodes.

        ::

            >>> a = rhythmtreetools.RhythmTreeContainer()
            >>> b = rhythmtreetools.RhythmTreeLeaf()
            >>> c = rhythmtreetools.RhythmTreeLeaf()

        ::

            >>> a.append(b)
            >>> b.parent is a
            True

        ::

            >>> a.children == (b,)
            True

        ::

            >>> a[0] = c

        ::

            >>> c.parent is a
            True

        ::

            >>> b.parent is None
            True

        ::

            >>> a.children == (c,)
            True

        Return `None`.
        '''
        from abjad.tools.rhythmtreetools.RhythmTreeParser \
            import RhythmTreeParser

        proper_parentage = self.proper_parentage

        if isinstance(i, int):
            if isinstance(expr, str):
                expr = RhythmTreeParser()(expr)[0]
                assert len(expr) == 1
                expr = expr[0]
            else:
                assert isinstance(expr, self._node_class)
            old = self[i]
            assert expr not in proper_parentage
            old._set_parent(None)
            expr._set_parent(self)
            self._children.insert(i, expr)
        else:
            if isinstance(expr, str):
                expr = RhythmTreeParser()(expr)
            elif isinstance(expr, list) and len(expr) == 1 and \
                isinstance(expr[0], str):
                expr = RhythmTreeParser()(expr[0])
            else:
                assert all(isinstance(x, self._node_class) for x in expr)
            if i.start == i.stop and i.start is not None \
                and i.stop is not None and i.start <= -len(self):
                start, stop = 0, 0
            else:
                start, stop, stride = i.indices(len(self))
            old = self[start:stop]
            for node in expr:
                assert node not in proper_parentage
            for node in old:
                node._set_parent(None)
            for node in expr:
                node._set_parent(self)
            self._children.__setitem__(slice(start, start), expr)
        self._mark_entire_tree_for_later_update()

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_duration(self):
        r'''The total preprolated_duration of the children 
        of a `RhythmTreeContainer` instance:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree._contents_duration
            Duration(5, 1)

        ::

            >>> tree[1]._contents_duration
            Duration(3, 1)

        Returns int.
        '''
        return sum([x.preprolated_duration for x in self])

    @property
    def _leaf_class(self):
        from abjad.tools import rhythmtreetools
        return rhythmtreetools.RhythmTreeLeaf

    @property
    def _node_class(self):
        return RhythmTreeNode

    @property
    def _pretty_rtm_format_pieces(self):
        result = []
        result.append('({!s} ('.format(self.preprolated_duration))
        for child in self:
            result.extend(['\t' + x for x in child._pretty_rtm_format_pieces])
        result[-1] = result[-1] + '))'
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def graphviz_graph(self):
        r'''The GraphvizGraph representation of the RhythmTreeContainer:

        ::

            >>> rtm = '(1 (1 (2 (1 1 1)) 2))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> graph = tree.graphviz_graph
            >>> print graph.graphviz_format
            digraph G {
                node_0 [label=1,
                    shape=triangle];
                node_1 [label=1,
                    shape=box];
                node_2 [label=2,
                    shape=triangle];
                node_3 [label=1,
                    shape=box];
                node_4 [label=1,
                    shape=box];
                node_5 [label=1,
                    shape=box];
                node_6 [label=2,
                    shape=box];
                node_0 -> node_1;
                node_0 -> node_2;
                node_0 -> node_6;
                node_2 -> node_3;
                node_2 -> node_4;
                node_2 -> node_5;
            }

        ::

            >>> topleveltools.graph(graph) # doctest: +SKIP

        Return `GraphvizGraph` instance.
        '''

        graph = documentationtools.GraphvizGraph(name='G')
        node_mapping = {}
        for node in self.nodes:
            graphviz_node = documentationtools.GraphvizNode()
            graphviz_node.attributes['label'] = str(node.preprolated_duration)
            if isinstance(node, type(self)):
                graphviz_node.attributes['shape'] = 'triangle'
            else:
                graphviz_node.attributes['shape'] = 'box'
            graph.append(graphviz_node)
            node_mapping[node] = graphviz_node
            if node.parent is not None:
                documentationtools.GraphvizEdge()(
                    node_mapping[node.parent],
                    node_mapping[node],
                    )
        return graph

    @property
    def rtm_format(self):
        r'''The node's RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> tree.rtm_format
            '(1 ((1 (1 1)) (1 (1 1))))'

        Returns string.
        '''
        return '({!s} ({}))'.format(
            self.preprolated_duration,
            ' '.join([x.rtm_format for x in self]))



# -*- encoding: utf-8 -*-
import numbers
from abjad.tools import mathtools


def repeat_sequence_to_weight_at_least(sequence, weight):
    '''Repeat `sequence` to `weight` at least:

    ::

        >>> sequencetools.repeat_sequence_to_weight_at_least((5, -5, -5), 23)
        (5, -5, -5, 5, -5)

    Returns newly constructed `sequence` object.
    '''

    # check input
    assert isinstance(weight, numbers.Number)
    assert 0 <= weight

    # initialize result
    result = [sequence[0]]

    # iterate input
    i = 1
    while mathtools.weight(result) < weight:
        result.append(sequence[i % len(sequence)])
        i += 1

    # return result
    return type(sequence)(result)



# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools


def list_unordered_named_pitch_pairs_in_expr(expr):
    '''List unordered named pitch pairs in `expr`:

    ::

        >>> chord = Chord("<c' cs' d' ef'>4")

    ::

        >>> for pair in pitchtools.list_unordered_named_pitch_pairs_in_expr(chord):
        ...     pair
        ...
        (NamedPitch("c'"), NamedPitch("cs'"))
        (NamedPitch("c'"), NamedPitch("d'"))
        (NamedPitch("c'"), NamedPitch("ef'"))
        (NamedPitch("cs'"), NamedPitch("d'"))
        (NamedPitch("cs'"), NamedPitch("ef'"))
        (NamedPitch("d'"), NamedPitch("ef'"))

    Returns generator.
    '''
    from abjad.tools import pitchtools

    for pair in sequencetools.yield_all_unordered_pairs_of_sequence(
        sorted(pitchtools.list_named_pitches_in_expr(expr))):
        yield pair



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_TextScriptSpanner___init___01():
    r'''Initialize empty text script spanner.
    '''

    spanner = spannertools.TextScriptSpanner()
    assert isinstance(spanner, spannertools.TextScriptSpanner)



# -*- encoding: utf-8 -*-
import abc
from abjad.tools.abctools import AbjadObject


class NonattributedBlock(list, AbjadObject):
    r'''Abjad model of LilyPond input file block with no attributes.
    '''

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats nonattributed block.

        Returns string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)

    def __repr__(self):
        r'''Gets interpreter representation of nonattributed block.

        Returns string.
        '''
        if not len(self):
            return '{}()'.format(type(self).__name__)
        else:
            return '{}({})'.format(type(self).__name__, len(self))

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        result = []
        if not len(self):
            if self._is_formatted_when_empty:
                result.append(r'%s {}' % self._escaped_name)
        else:
            result.append(r'%s {' % self._escaped_name)
            for x in self:
                if hasattr(x, '_format_pieces'):
                    result.extend(['\t' + piece for piece in x._format_pieces])
                elif isinstance(x, str):
                    result.append('\t%s' % x)
            result.append('}')
        return result

    @property
    def _lilypond_format(self):
        return '\n'.join(self._format_pieces)

    ### PUBLIC PROPERTIES ###

    @property
    def is_formatted_when_empty(self):
        r'''True when nonattributed block is formatted when empty. Otherwise
        false.

        Returns boolean.
        '''
        return self._is_formatted_when_empty

    @is_formatted_when_empty.setter
    def is_formatted_when_empty(self, arg):
        if isinstance(arg, bool):
            self._is_formatted_when_empty = arg
        else:
            raise TypeError



# -*- encoding: utf-8 -*-
from abjad.tools import *


def test_sequencetools_partition_sequence_by_backgrounded_weights_01():

    assert sequencetools.partition_sequence_by_backgrounded_weights(
        [-5, -15, -10], [20, 10]) == [[-5, -15], [-10]]

    assert sequencetools.partition_sequence_by_backgrounded_weights(
        [-5, -15, -10], [5, 5, 5, 5, 5, 5]) == [[-5], [-15], [], [], [-10], []]

    assert sequencetools.partition_sequence_by_backgrounded_weights(
        [-5, -15, -10], [1, 29]) == [[-5], [-15, -10]]

    assert sequencetools.partition_sequence_by_backgrounded_weights(
        [-5, -15, -10], [2, 28]) == [[-5], [-15, -10]]

    assert sequencetools.partition_sequence_by_backgrounded_weights(
        [-5, -15, -10], [1, 1, 1, 1, 1, 25]) == [[-5], [], [], [], [], [-15, -10]]



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_partition_sequence_by_weights_exactly_01():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights_exactly(
        sequence, [3, 9], cyclic=False, overhang=True)
    assert groups == [[3], [3, 3, 3], [4, 4, 4, 4, 5, 5]]


def test_sequencetools_partition_sequence_by_weights_exactly_02():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights_exactly(
        sequence, [3, 9], cyclic=False, overhang=False)
    assert groups == [[3], [3, 3, 3]]


def test_sequencetools_partition_sequence_by_weights_exactly_03():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]
    groups = sequencetools.partition_sequence_by_weights_exactly(
        sequence, [12], cyclic=True, overhang=True)
    assert groups == [[3, 3, 3, 3], [4, 4, 4], [4, 5]]


def test_sequencetools_partition_sequence_by_weights_exactly_04():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]
    groups = sequencetools.partition_sequence_by_weights_exactly(
        sequence, [12], cyclic=True, overhang=False)
    assert groups == [[3, 3, 3, 3], [4, 4, 4]]



# -*- encoding: utf-8 -*-
# TODO: implement subsegment containment on pitchtools Segment classes
def contains_subsegment(pitch_class_numbers, pitch_numbers):
    '''True when `pitch_numbers` contain `pitch_class_numbers`
    as subsegment:

    ::

        >>> pcs = [2, 7, 10]
        >>> pitches = [6, 9, 12, 13, 14, 19, 22, 27, 28, 29, 32, 35]
        >>> pitchtools.contains_subsegment(pcs, pitches)
        True

    Returns boolean.
    '''

    pcs_start_index = [p % 12 for p in pitch_numbers].index(
        pitch_class_numbers[0] % 12)
    pcs_transposition = pitch_numbers[pcs_start_index] - pitch_class_numbers[0]
    transposed_pcs = [p + pcs_transposition for p in pitch_class_numbers]
    return set(transposed_pcs).issubset(set(pitch_numbers))



# -*- encoding: utf-8 -*-
from abjad import *


def test_stringtools_is_upper_camel_case_string_01():

    assert stringtools.is_upper_camel_case_string('FooBar')
    assert stringtools.is_upper_camel_case_string('Foo')
    assert stringtools.is_upper_camel_case_string('FB')
    assert stringtools.is_upper_camel_case_string('F')


def test_stringtools_is_upper_camel_case_string_02():

    assert stringtools.is_upper_camel_case_string('')


def test_stringtools_is_upper_camel_case_string_03():

    assert not stringtools.is_upper_camel_case_string('fooBar')
    assert not stringtools.is_upper_camel_case_string('foobar')
    assert not stringtools.is_upper_camel_case_string('foo bar')
    assert not stringtools.is_upper_camel_case_string('Foo Bar')
    assert not stringtools.is_upper_camel_case_string('fb')



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class StorageFormatSpecification(AbjadObject):
    r'''Specifies the storage format of a given object.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_body_text',
        '_instance',
        '_is_bracketted',
        '_is_indented',
        '_keyword_argument_names',
        '_keywords_ignored_when_false',
        '_positional_argument_values',
        '_storage_format_pieces',
        '_tools_package_name',
        )

    ### INITIALIZER ###

    def __init__(self,
        instance=None,
        body_text=None,
        is_bracketted=False,
        is_indented=True,
        keyword_argument_names=None,
        keywords_ignored_when_false=None,
        positional_argument_values=None,
        storage_format_pieces=None,
        tools_package_name=None,
        ):
        self._instance = instance
        if body_text is not None:
            self._body_text = str(body_text)
        else:
            self._body_text = None
        self._is_bracketted = bool(is_bracketted)
        self._is_indented = bool(is_indented)
        if keyword_argument_names is not None:
            self._keyword_argument_names = tuple(keyword_argument_names)
        else:
            self._keyword_argument_names = None
        if keywords_ignored_when_false is not None:
            self._keywords_ignored_when_false = \
                tuple(keywords_ignored_when_false)
        else:
            self._keywords_ignored_when_false = None
        if positional_argument_values is not None:
            self._positional_argument_values = \
                tuple(positional_argument_values)
        else:
            self._positional_argument_values = None
        if storage_format_pieces is not None:
            self._storage_format_pieces = tuple(storage_format_pieces)
        else:
            self._storage_format_pieces = None
        if tools_package_name is not None:
            self._tools_package_name = str(tools_package_name)
        else:
            self._tools_package_name = None

    ### SPECIAL METHODS ###

    def __makenew__(self, *args, **kwargs):
        r'''Makes new storage format specification with optional `kwargs`.

        Returns new storage format specification.
        '''
        state = {}
        for slot in self.__slots__:
            name = slot[1:]
            state[name] = kwargs.get(name, getattr(self, slot))
        return type(self)(**state)

    ### PUBLIC PROPERTIES ###

    @property
    def body_text(self):
        r'''Body text of storage specification.

        Returns string.
        '''
        return self._body_text

    @property
    def instance(self):
        r'''Instance of storage specification.

        Returns string.
        '''
        return self._instance

    @property
    def is_bracketted(self):
        r'''True when storage specification is bracketted.
        Otherwise false.

        Returns boolean.
        '''
        return self._is_bracketted

    @property
    def is_indented(self):
        r'''True when storage format is indented. Otherwise false.

        Returns boolean.
        '''
        return self._is_indented

    @property
    def keyword_argument_names(self):
        r'''Keyword argument names of storage format.

        Returns tuple.
        '''
        from abjad.tools import systemtools
        if self._keyword_argument_names is not None:
            names = self._keyword_argument_names
        else:
            names = \
                systemtools.StorageFormatManager.get_keyword_argument_names(
                    self.instance)
        if self._keywords_ignored_when_false is not None:
            names = list(names)
            for name in self._keywords_ignored_when_false:
                result = getattr(self.instance, name, True)
                if not result and name in names:
                    names.remove(name)
            return tuple(names)
        return names

    @property
    def keywords_ignored_when_false(self):
        r'''Keywords ignored when false.

        Returns tuple.
        '''
        return self._keywords_ignored_when_false

    @property
    def positional_argument_values(self):
        r'''Positional argument values.

        Returns tuple.
        '''
        from abjad.tools import systemtools
        if self._positional_argument_values is not None:
            return self._positional_argument_values
        return systemtools.StorageFormatManager.get_positional_argument_values(
            self.instance)

    @property
    def storage_format_pieces(self):
        r'''Storage format pieces.

        Returns tuple.
        '''
        return self._storage_format_pieces

    @property
    def tools_package_name(self):
        r'''Tools package name of storage format.

        Returns string.
        '''
        from abjad.tools import systemtools
        if self._tools_package_name is not None:
            return self._tools_package_name
        if self.instance is not None:
            return systemtools.StorageFormatManager.get_tools_package_name(
                self.instance)



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.rhythmmakertools.BurnishedRhythmMaker \
	import BurnishedRhythmMaker


class OutputBurnishedTaleaRhythmMaker(BurnishedRhythmMaker):
    r'''Output-burnished talea rhythm-maker:

    ::

        >>> maker = rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
        ...     talea=[1, 2, 3],
        ...     talea_denominator=16,
        ...     prolation_addenda=[0, 2],
        ...     lefts=[-1],
        ...     middles=[0],
        ...     rights=[-1],
        ...     left_lengths=[1],
        ...     right_lengths=[1],
        ...     secondary_divisions=[9],
        ...     beam_each_cell=True)

    Configure at initialization and then call on any list of divisions:

    ::

        >>> divisions = [(3, 8), (4, 8)]
        >>> music = maker(divisions)
        >>> music = sequencetools.flatten_sequence(music)
        >>> measures = \
        ...     scoretools.make_spacer_skip_measures(
        ...     divisions)
        >>> staff = scoretools.RhythmicStaff(measures)
        >>> measures = scoretools.replace_contents_of_measures_in_expr(
        ...     staff, music)

    ::

        >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats output-burnished talea rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
                talea=[1, 2, 3],
                talea_denominator=16,
                prolation_addenda=[0, 2],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[9],
                beam_each_cell=True,
                beam_cells_together=False,
                decrease_durations_monotonically=True,
                tie_split_notes=False,
                tie_rests=False,
                )

        Returns string.
        '''
        superclass = super(OutputBurnishedTaleaRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new output-burnished talea rhythm-maker with `kwargs`.

        ::

            >>> new_maker = new(maker, secondary_divisions=[10])

        ::

            >>> print format(new_maker)
            rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
                talea=[1, 2, 3],
                talea_denominator=16,
                prolation_addenda=[0, 2],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[10],
                beam_each_cell=True,
                beam_cells_together=False,
                decrease_durations_monotonically=True,
                tie_split_notes=False,
                tie_rests=False,
                )

        ::

            >>> divisions = [(3, 8), (4, 8)]
            >>> music = new_maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new output-burnished talea rhythm-maker.
        '''
        return BurnishedRhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _burnish_division_parts(self, divisions, quintuplet):
        lefts, middles, rights, left_lengths, right_lengths = quintuplet
        burnished_divisions = []
        left_length = left_lengths[0]
        left = lefts[:left_length]
        right_length = right_lengths[0]
        right = rights[:right_length]
        if len(divisions) == 1:
            available_left_length = len(divisions[0])
            left_length = min([left_length, available_left_length])
            available_right_length = len(divisions[0]) - left_length
            right_length = min([right_length, available_right_length])
            middle_length = len(divisions[0]) - left_length - right_length
            left = left[:left_length]
            middle = middle_length * [middles[0]]
            right = right[:right_length]
            left_part, middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                divisions[0],
                [left_length, middle_length, right_length], 
                cyclic=False,
                overhang=False)
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = left_part + middle_part + right_part
            burnished_divisions.append(burnished_division)
        else:
            ## first division
            available_left_length = len(divisions[0])
            left_length = min([left_length, available_left_length])
            middle_length = len(divisions[0]) - left_length
            left = left[:left_length]
            middle = middle_length * [middles[0]]
            left_part, middle_part = \
                sequencetools.partition_sequence_by_counts(
                divisions[0], 
                [left_length, middle_length],
                cyclic=False,
                overhang=False)
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            burnished_division = left_part + middle_part
            burnished_divisions.append(burnished_division)
            ## middle divisions
            for division in divisions[1:-1]:
                middle_part = division
                middle = len(division) * [middles[0]]
                middle_part = self._burnish_division_part(middle_part, middle)
                burnished_division = middle_part
                burnished_divisions.append(burnished_division)
            ## last division:
            available_right_length = len(divisions[-1])
            right_length = min([right_length, available_right_length])
            middle_length = len(divisions[-1]) - right_length
            right = right[:right_length]
            middle = middle_length * [middles[0]]
            middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                divisions[-1], 
                [middle_length, right_length], 
                cyclic=False,
                overhang=False)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = middle_part + right_part
            burnished_divisions.append(burnished_division)
        unburnished_weights = [mathtools.weight(x) for x in divisions]
        burnished_weights = [mathtools.weight(x) for x in burnished_divisions]
        assert burnished_weights == unburnished_weights
        return burnished_divisions

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses output-burnished talea rhythm-maker:

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
                talea=[3, 2, 1],
                talea_denominator=16,
                prolation_addenda=[2, 0],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[9],
                beam_each_cell=True,
                beam_cells_together=False,
                decrease_durations_monotonically=False,
                tie_split_notes=False,
                tie_rests=False,
                )

        ::

            >>> divisions = [(3, 8), (4, 8)]
            >>> music = reversed_maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new output-burnished talea rhythm-maker.
        '''
        return BurnishedRhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRangeInventory___format___01():

    inventory = pitchtools.PitchRangeInventory(['[A0, C8]', '[C4, D5]'])

    assert systemtools.TestManager.compare(
        format(inventory),
        r'''
        pitchtools.PitchRangeInventory(
            [
                pitchtools.PitchRange(
                    '[A0, C8]'
                    ),
                pitchtools.PitchRange(
                    '[C4, D5]'
                    ),
                ]
            )
        ''',
        )



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def notes_and_chords_are_in_range(expr):
    '''True when notes and chords in `expr` are 
    within traditional instrument ranges.

    ::

        >>> staff = Staff("c'8 r8 <d' fs'>8 r8")
        >>> violin = instrumenttools.Violin()
        >>> attach(violin, staff)

    ::

        >>> instrumenttools.notes_and_chords_are_in_range(staff)
        True

    Otherwise false:

    ::

        >>> staff = Staff("c'8 r8 <d fs>8 r8")
        >>> violin = instrumenttools.Violin()
        >>> attach(violin, staff)

    ::

        >>> instrumenttools.notes_and_chords_are_in_range(staff)
        False

    Returns boolean.
    '''
    from abjad.tools import instrumenttools

    for note_or_chord in iterate(expr).by_class(
        (scoretools.Note, scoretools.Chord)):
        instrument = note_or_chord._get_effective(
            instrumenttools.Instrument)
        if note_or_chord not in instrument.pitch_range:
            return False
    else:
        return True



# -*- encoding: utf-8 -*-
import abc
from abjad.tools.abctools import AbjadObject


class AttributedBlock(list, AbjadObject):
    '''Abjad model of LilyPond input file block with attributes.
    '''

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        self._is_formatted_when_empty = False

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats attributed block.

        Returns string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)

    def __repr__(self):
        r'''Gets interpreter representation of attributed block.

        Returns string.
        '''
        if not len(self._user_attributes):
            return '{}()'.format(type(self).__name__)
        else:
            return '{}({})'.format(
                type(self).__name__, 
                len(self._user_attributes),
                )

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        result = []
        if not self._formatted_user_attributes and \
            not getattr(self, 'contexts', None) \
            and not getattr(self, 'context_blocks', None):
            if self.is_formatted_when_empty:
                result.append('%s {}' % self._escaped_name)
                return result
            else:
                return result
        result.append('%s {' % self._escaped_name)
        if getattr(self, 'contexts', None):
            specs = self._formatted_context_specifications
            result.extend(['\t' + x for x in specs])
        formatted_attributes = self._formatted_user_attributes
        formatted_attributes = ['\t' + x for x in formatted_attributes]
        result.extend(formatted_attributes)
        formatted_context_blocks = getattr(
            self, '_formatted_context_blocks', [])
        formatted_context_blocks = [
            '\t' + line for line in formatted_context_blocks]
        result.extend(formatted_context_blocks)
        result.append('}')
        return result

    @property
    def _formatted_user_attributes(self):
        from abjad.tools import lilypondfiletools
        from abjad.tools import indicatortools
        from abjad.tools import markuptools
        from abjad.tools import schemetools
        result = []
        for value in self:
            acceptable_types = (
                schemetools.Scheme, 
                indicatortools.LilyPondCommand,
                )
            if isinstance(value, acceptable_types):
                result.append(format(value, 'lilypond'))
        for key, value in sorted(vars(self).items()):
            if not key.startswith('_'):
                # format subkeys via double underscore
                formatted_key = key.split('__')
                for i, k in enumerate(formatted_key):
                    formatted_key[i] = k.replace('_', '-')
                    if 0 < i:
                        formatted_key[i] = "#'%s" % formatted_key[i]
                formatted_key = ' '.join(formatted_key)
                # format value
                accetable_types = (
                    schemetools.Scheme,
                    lilypondfiletools.LilyPondDimension,
                    indicatortools.LilyPondCommand,
                    )
                if isinstance(value, markuptools.Markup):
                    formatted_value = value._get_format_pieces()
                elif isinstance(value, accetable_types):
                    formatted_value = [format(value, 'lilypond')]
                else:
                    formatted_value = schemetools.Scheme(value)
                    formatted_value = format(formatted_value, 'lilypond')
                    formatted_value = [formatted_value]
                setting = '{!s} = {!s}'
                setting = setting.format(
                    formatted_key, 
                    formatted_value[0],
                    )
                result.append(setting)
                result.extend(formatted_value[1:])
        return result

    @property
    def _lilypond_format(self):
        return '\n'.join(self._format_pieces)

    @property
    def _user_attributes(self):
        all_attributes = vars(self).keys()
        user_attributes = [x for x in all_attributes if not x.startswith('_')]
        user_attributes.sort()
        return user_attributes

    ### PUBLIC PROPERTIES ###

    @property
    def is_formatted_when_empty(self):
        r'''True when attributed block is formatted when empty. Otherwise
        false.

        Returns boolean.
        '''
        return self._is_formatted_when_empty

    @is_formatted_when_empty.setter
    def is_formatted_when_empty(self, arg):
        if isinstance(arg, bool):
            self._is_formatted_when_empty = arg
        else:
            raise TypeError



# -*- encoding: utf-8 -*-
import copy
import re
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools.scoretools.Leaf import Leaf
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach


class Chord(Leaf):
    r'''A chord.

    ..  container:: example

        ::

            >>> chord = Chord("<e' cs'' f''>4")
            >>> show(chord) # doctest: +SKIP

        ..  doctest::

            >>> print format(chord)
            <e' cs'' f''>4

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_note_heads',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import lilypondparsertools
        from abjad.tools import scoretools
        assert len(args) in (0, 1, 2)
        self._note_heads = scoretools.NoteHeadInventory(
            client=self,
            )
        if len(args) == 1 and isinstance(args[0], str):
            string = '{{ {} }}'.format(args[0])
            parsed = lilypondparsertools.LilyPondParser()(string)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            args = [parsed[0]]
        is_cautionary = []
        is_forced = []
        if len(args) == 1 and isinstance(args[0], Leaf):
            leaf = args[0]
            written_duration = leaf.written_duration
            if hasattr(leaf, 'written_pitch'):
                written_pitches = [leaf.written_pitch]
                is_cautionary = [leaf.note_head.is_cautionary]
                is_forced = [leaf.note_head.is_forced]
            elif hasattr(leaf, 'written_pitches'):
                written_pitches = leaf.written_pitches
                is_cautionary = [x.is_cautionary for x in leaf.note_heads]
                is_forced = [x.is_forced for x in leaf.note_heads]
            else:
                written_pitches = []
        elif len(args) == 2:
            written_pitches, written_duration = args
            if isinstance(written_pitches, str):
                written_pitches = [x for x in written_pitches.split() if x]
            elif isinstance(written_pitches, type(self)):
                written_pitches = written_pitches.written_pitches
        elif len(args) == 0:
            written_pitches = [0, 4, 7]
            written_duration = durationtools.Duration(1, 4)
        else:
            message = 'can not initialize chord from {!r}.'
            message = message.format(args)
            raise ValueError(message)
        Leaf.__init__(self, written_duration)
        if not is_cautionary:
            is_cautionary = [False] * len(written_pitches)
        if not is_forced:
            is_forced = [False] * len(written_pitches)
        for written_pitch, cautionary, forced in zip(
            written_pitches, is_cautionary, is_forced):
            note_head = scoretools.NoteHead(
                written_pitch=written_pitch,
                is_cautionary=cautionary,
                is_forced=forced,
                )
            self._note_heads.append(note_head)
        if len(args) == 1 and isinstance(args[0], Leaf):
            self._copy_override_and_set_from_leaf(args[0])

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        r'''Gets new arguments.
    
        Returns tuple.
        '''
        return (self.written_pitches, self.written_duration)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        return '<{}>{}'.format(self._summary, self._formatted_duration)

    @property
    def _lilypond_format(self):
        return super(Chord, self)._lilypond_format

    @property
    def _summary(self):
        return ' '.join([str(x) for x in self.note_heads])

    ### PRIVATE METHODS ###

    @staticmethod
    def _cast_defective_chord(chord):
        from abjad.tools import scoretools
        if isinstance(chord, Chord):
            note_head_count = len(chord.note_heads)
            if not note_head_count:
                return scoretools.Rest(chord)
            elif note_head_count == 1:
                return scoretools.Note(chord)
        return chord

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Leaf._copy_with_indicators_but_without_children_or_spanners(self)
        new.note_heads[:] = []
        for note_head in self.note_heads:
            new_note_head = copy.copy(note_head)
            new.note_heads.append(new_note_head)
        return new

    def _divide(self, pitch=None):
        from abjad.tools import scoretools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        pitch = pitch or pitchtools.NamedPitch('b', 3)
        pitch = pitchtools.NamedPitch(pitch)
        treble = copy.copy(self)
        bass = copy.copy(self)
        detach(markuptools.Markup, treble)
        detach(markuptools.Markup, bass)

        if isinstance(treble, scoretools.Note):
            if treble.written_pitch < pitch:
                treble = scoretools.Rest(treble)
        elif isinstance(treble, scoretools.Rest):
            pass
        elif isinstance(treble, scoretools.Chord):
            for note_head in reversed(treble.note_heads):
                if note_head.written_pitch < pitch:
                    treble.note_heads.remove(note_head)
        else:
            raise TypeError

        if isinstance(bass, scoretools.Note):
            if pitch <= bass.written_pitch:
                bass = scoretools.Rest(bass)
        elif isinstance(bass, scoretools.Rest):
            pass
        elif isinstance(bass, scoretools.Chord):
            for note_head in reversed(bass.note_heads):
                if pitch <= note_head.written_pitch:
                    bass.note_heads.remove(note_head)
        else:
            raise TypeError

        treble = self._cast_defective_chord(treble)
        bass = self._cast_defective_chord(bass)

        up_markup = self._get_markup(direction=Up)
        up_markup = [copy.copy(markup) for markup in up_markup]

        down_markup = self._get_markup(direction=Down)
        down_markup = [copy.copy(markup) for markup in down_markup]

        for markup in up_markup:
            markup(treble)
        for markup in down_markup:
            markup(bass)

        return treble, bass

    def _get_sounding_pitches(self):
        from abjad.tools import instrumenttools
        from abjad.tools import pitchtools
        if self._has_effective_indicator(indicatortools.IsAtSoundingPitch):
            return self.written_pitches
        else:
            instrument = self._get_effective(
                instrumenttools.Instrument)
            if instrument:
                sounding_pitch = instrument.sounding_pitch_of_written_middle_c
            else:
                sounding_pitch = pitchtools.NamedPitch('C4')
            interval = pitchtools.NamedPitch('C4') - sounding_pitch
            sounding_pitches = [
                pitchtools.transpose_pitch_carrier_by_interval(
                pitch, interval) for pitch in self.written_pitches]
            return tuple(sounding_pitches)

    ### PUBLIC PROPERTIES ###

    @property
    def note_heads(self):
        r'''Note heads in chord.

        ..  container:: example

            **Example 1.** Get note heads in chord:

            ::

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> print format(chord.note_heads)
                scoretools.NoteHeadInventory(
                    [
                        scoretools.NoteHead(
                            written_pitch=pitchtools.NamedPitch("g'"),
                            is_cautionary=False,
                            is_forced=False,
                            ),
                        scoretools.NoteHead(
                            written_pitch=pitchtools.NamedPitch("c''"),
                            is_cautionary=False,
                            is_forced=False,
                            ),
                        scoretools.NoteHead(
                            written_pitch=pitchtools.NamedPitch("e''"),
                            is_cautionary=False,
                            is_forced=False,
                            ),
                        ]
                    )

        ..  container:: example

            **Example 2.** Set note heads with pitch names:

            ::

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.note_heads = "c' d' fs'"
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print format(chord)
                <c' d' fs'>4

        ..  container:: example

            **Example 3.** Set note heads with pitch numbers:

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.note_heads = [16, 17, 19]
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print format(chord)
                <e'' f'' g''>4

        Set note heads with any iterable.

        Returns tuple.
        '''
        return self._note_heads

    @note_heads.setter
    def note_heads(self, note_heads):
        self._note_heads[:] = []
        if isinstance(note_heads, str):
            note_heads = note_heads.split()
        self.note_heads.extend(note_heads)

    @property
    def written_duration(self):
        r'''Written duration of chord.

        ..  container:: example

            **Example 1.** Get written duration:

            ::

                >>> chord = Chord("<e' cs'' f''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_duration
                Duration(1, 4)

        ..  container:: example

            **Example 2.** Set written duration:

            ::

                >>> chord = Chord("<e' cs'' f''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_duration = Duration(1, 16)
                >>> show(chord) # doctest: +SKIP

        Set duration.

        Returns duration.
        '''
        return Leaf.written_duration.fget(self)

    @written_duration.setter
    def written_duration(self, expr):
        Leaf.written_duration.fset(self, expr)

    @property
    def written_pitches(self):
        r'''Written pitches in chord.

        ..  container:: example

            **Example 1.** Get written pitches:

                >>> chord = Chord("<g' c'' e''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> for written_pitch in chord.written_pitches:
                ...     written_pitch
                NamedPitch("g'")
                NamedPitch("c''")
                NamedPitch("e''")

        ..  container:: example

            **Example 2.** Set written pitches with pitch names:

            ::

                >>> chord = Chord("<e' g' c''>4")
                >>> show(chord) # doctest: +SKIP

            ::

                >>> chord.written_pitches = "f' b' d''"
                >>> show(chord) # doctest: +SKIP

            ..  doctest::

                >>> print format(chord)
                <f' b' d''>4

        Set written pitches with any iterable.

        Returns tuple.
        '''
        return tuple(note_head.written_pitch
            for note_head in self.note_heads)

    @written_pitches.setter
    def written_pitches(self, pitches):
        self.note_heads = pitches



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_list_pitch_numbers_in_expr_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    assert pitchtools.list_pitch_numbers_in_expr(tuplet) == (0, 2, 4)


def test_pitchtools_list_pitch_numbers_in_expr_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    assert pitchtools.list_pitch_numbers_in_expr(staff) == (0, 2, 4, 5)



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_Markup___init___01():
    r'''Initialize markup with string.
    '''

    markup = markuptools.Markup('foo')
    assert str(markup) == '\\markup { foo }'


def test_markuptools_Markup___init___02():
    r'''Initialize markup with other markup instance.
    '''

    markup_1 = markuptools.Markup('foo')
    markup_2 = markuptools.Markup(markup_1)

    assert str(markup_1) == '\\markup { foo }'
    assert str(markup_2) == '\\markup { foo }'


def test_markuptools_Markup___init___03():
    r'''Initialize markup with nonstring and nonmarkup instance.
    '''

    markup = markuptools.Markup(27)
    assert str(markup) == '\\markup { 27 }'



def test_markuptools_Markup___init___04():
    r'''Initialize markup from MarkupCommand.
    '''

    command = markuptools.MarkupCommand('flat')
    markup = markuptools.Markup(command)
    assert systemtools.TestManager.compare(
        str(markup),
        r'''
        \markup {
            \flat
            }
        ''',
        )


def test_markuptools_Markup___init___05():
    r'''Initialize markup from sequence of strings or MarkupCommands.
    '''

    command_1 = markuptools.MarkupCommand('flat')
    command_2 = markuptools.MarkupCommand('sharp')
    markup = markuptools.Markup(['X', command_1, 'Y', command_2, 'Z'])
    assert systemtools.TestManager.compare(
        str(markup),
        r'''
        \markup {
            X
            \flat
            Y
            \sharp
            Z
            }
        ''',
        )

def test_markuptools_Markup___init___06():
    r'''Preserve all keywords when initializing from other markup instance.
    '''

    markup_1 = markuptools.Markup(
        'foo contents string', direction=Up, markup_name='foo')
    markup_2 = markuptools.Markup(markup_1)

    assert markup_1 is not markup_2
    assert markup_1 == markup_2
    assert repr(markup_1) == repr(markup_2)
    assert format(markup_1) == format(markup_2)


def test_markuptools_Markup___init___07():
    r'''Preserve keywords when initializing from other markup instance
    but also overwrite keywords specified anew.
    '''

    markup_1 = markuptools.Markup(
        'foo contents string', direction=Up, markup_name='foo')
    markup_2 = markuptools.Markup(markup_1, direction=Down)

    assert markup_1 is not markup_2
    assert markup_2.contents == ('foo contents string',)
    assert markup_2.direction is Down
    assert markup_2.markup_name == 'foo'



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_make_time_signature_context_block_01():

    context_block = lilypondfiletools.make_time_signature_context_block()

    assert systemtools.TestManager.compare(
        context_block,
        r'''
        \context {
            \name TimeSignatureContext
            \type Engraver_group
            \consists Axis_group_engraver
            \consists Time_signature_engraver
            \override TimeSignature #'X-extent = #'(0 . 0)
            \override TimeSignature #'X-offset = #ly:self-alignment-interface::x-aligned-on-self
            \override TimeSignature #'Y-extent = #'(0 . 0)
            \override TimeSignature #'break-align-symbol = ##f
            \override TimeSignature #'break-visibility = #end-of-line-invisible
            \override TimeSignature #'font-size = #3
            \override TimeSignature #'self-alignment-X = #center
            \override VerticalAxisGroup #'default-staff-staff-spacing = #'((basic-distance . 0) (minimum-distance . 12) (padding . 4) (stretchability . 0))
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Hairpin import Hairpin


class Crescendo(Hairpin):
    r'''A crescendo spanner that includes rests.

    ::

        >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            r4
            c'8
            d'8
            e'8
            f'8
            r4
        }

    ::

        >>> crescendo = spannertools.Crescendo(include_rests=True)
        >>> attach(crescendo, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            r4 \<
            c'8
            d'8
            e'8
            f'8
            r4 \!
        }

    Abjad crescendo spanner that does not include rests:

    ::

        >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            r4
            c'8
            d'8
            e'8
            f'8
            r4
        }

    ::

        >>> crescendo = spannertools.Crescendo(include_rests=False)
        >>> attach(crescendo, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            r4
            c'8 \<
            d'8
            e'8
            f'8 \!
            r4
        }

    '''

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None, 
        include_rests=True, 
        direction=None,
        overrides=None,
        ):
        Hairpin.__init__(
            self, components=components, 
            descriptor='<', 
            include_rests=include_rests,
            direction=direction,
            overrides=overrides,
            )



# -*- encoding: utf-8 -*-
import abc
import configobj
import os
import time
import validate
from abjad.tools.abctools.AbjadObject import AbjadObject


class Configuration(AbjadObject):
    r'''A configuration object.
    '''

    ### INITIALIZER ###

    def __init__(self):
        # verify configuration directory
        if not os.path.exists(self.configuration_directory_path):
            os.makedirs(self.configuration_directory_path)
        # attempt to load config from disk, and validate
        # a config object will be created if none is found on disk
        config = configobj.ConfigObj(
            self.configuration_file_path,
            configspec=self._config_specification
            )
        # validate
        validator = validate.Validator()
        validation = config.validate(validator, copy=True)
        # replace failing key:value pairs with default values
        if validation is not True:
            for key, valid in validation.iteritems():
                if not valid:
                    default = config.default_values[key]
                    message = 'Warning: config key {!r} failed validation,'
                    message += ' setting to default: {!r}.'
                    message = message.format(key, default)
                    print message
                    config[key] = default
        # setup output formatting
        config.write_empty_values = True
        config.comments.update(self._option_comments)
        config.initial_comment = self._initial_comment
        # write back to disk if different
        config.filename = None  # prevent ConfigObj from automatically writing
        with open(self.configuration_file_path, 'r') as f:
            old_config_lines = f.read().splitlines()
        while len(old_config_lines) and (
            old_config_lines[0].startswith('#') or
            not old_config_lines[0].strip()):
            old_config_lines.pop(0)
        new_config_lines = config.write(None)
        while len(new_config_lines) and (
            new_config_lines[0].startswith('#') or
            not new_config_lines[0].strip()):
            new_config_lines.pop(0)
        if old_config_lines != new_config_lines:
            with open(self.configuration_file_path, 'w') as f:
                config.write(f)
        # turn the ConfigObj instance into a standard dict,
        # and replace its empty string values with Nones,
        # caching the result on this AbjadConfiguration instance.
        self._settings = dict(config)
        for key, value in self._settings.iteritems():
            if value == '' or value == 'None':
                self._settings[key] = None

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        r'''Deletes `i` from settings.

        Returns none.
        '''
        del(self._settings[i])

    def __getitem__(self, i):
        r'''Gets `i` from settings.

        Returns none.
        '''
        return self._settings[i]

    def __iter__(self):
        r'''Iterates settings.

        Returns generator.
        '''
        for key in self._settings:
            yield key

    def __len__(self):
        r'''Number of settings in configuration.

        Returns nonnegative integer.
        '''
        return len(self._settings)

    def __setitem__(self, i, arg):
        r'''Sets setting `i` to `arg`.

        Returns none.
        '''
        self._settings[i] = arg

    ### PRIVATE PROPERTIES ###

    @property
    def _config_specification(self):
        specs = self._option_specification
        return ['{} = {}'.format(key, value) 
            for key, value in sorted(specs.items())]

    @property
    def _current_time(self):
        return time.strftime("%d %B %Y %H:%M:%S")

    @abc.abstractproperty
    def _initial_comment(self):
        raise NotImplemented

    @property
    def _option_comments(self):
        options = self._option_definitions
        comments = [(key, options[key]['comment']) for key in options]
        return dict(comments)

    @abc.abstractproperty
    def _option_definitions(self):
        raise NotImplemented

    @property
    def _option_specification(self):
        options = self._option_definitions
        specs = [(key, options[key]['spec']) for key in options]
        return dict(specs)

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def configuration_directory_path(self):
        r'''Configuration directory path.

        Returns string.
        '''
        raise NotImplemented

    @abc.abstractproperty
    def configuration_file_name(self):
        r'''Configuration file name.

        Returns string.
        '''
        raise NotImplemented

    @property
    def configuration_file_path(self):
        r'''Configuration file path.

        Returns string.
        '''
        return os.path.join(
            self.configuration_directory_path, 
            self.configuration_file_name,
            )

    @property
    def home_directory_path(self):
        r'''Home directory path.

        Returns string.
        '''
        path = os.environ.get('HOME') or \
            os.environ.get('HOMEPATH') or \
            os.environ.get('APPDATA')
        return os.path.abspath(path)



# -*- encoding: utf-8 -*-
import copy
from abjad.tools.abctools import AbjadObject


class TreeNode(AbjadObject):
    r'''A node.
    
    Node in a generalized tree.
    '''

    ### INITIALIZER ###

    def __init__(self, name=None):
        self._name = None
        self._parent = None
        self.name = name

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies tree node.

        Returns new tree node.
        '''
        return type(self)(
            *[copy.deepcopy(x) for x in self.__getnewargs__()]
            )

    # TODO: remove? we shouldn't alias deepcopy anywhere
    __deepcopy__ = __copy__

    def __eq__(self, expr):
        r'''True when `expr` is a tree node. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            return True
        return False

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.get_input_argument_values(
            self)

    def __getstate__(self):
        r'''Gets state.

        Returns dictionary.
        '''
        state = {}
        class_dir = set(dir(type(self)))
        self_dir = set(x for x in dir(self) if x.startswith('_') and
            not x.startswith('__'))
        for name in self_dir.difference(class_dir):
            state[name] = getattr(self, name)
        return state

    def __ne__(self, expr):
        r'''True when tree node does not equal `expr`. Otherwise false.

        Returns boolean.
        '''
        return not self.__eq__(expr)

    def __setstate__(self, state):
        r'''Sets `state`.

        Returns none.
        '''
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### PRIVATE METHODS ###

    def _get_node_state_flags(self):
        state_flags = {}
        for name in self._state_flag_names:
            state_flags[name] = True
            for node in self.improper_parentage:
                if not getattr(node, name):
                    state_flags[name] = False
                    break
        return state_flags

    def _mark_entire_tree_for_later_update(self):
        for node in self.improper_parentage:
            for name in self._state_flag_names:
                setattr(node, name, False)

    def _set_parent(self, new_parent):
        name_dictionary = {}
        if hasattr(self, '_named_children'):
            for name, children in self._named_children.iteritems():
                name_dictionary[name] = copy.copy(children)
        if hasattr(self, 'name') and self.name is not None:
            if self.name not in name_dictionary:
                name_dictionary[self.name] = set([])
            name_dictionary[self.name].add(self)
        if self._parent is not None and name_dictionary:
            for parent in self.proper_parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    for node in name_dictionary[name]:
                        named_children[name].remove(node)
                    if not named_children[name]:
                        del named_children[name]
        if self._parent is not None:
            index = self._parent.index(self)
            self._parent._children.pop(index)
        self._parent = new_parent
        if new_parent is not None and name_dictionary:
            for parent in self.proper_parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    if name in named_children:
                        named_children[name].update(name_dictionary[name])
                    else:
                        named_children[name] = copy.copy(name_dictionary[name])
        self._mark_entire_tree_for_later_update()

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification

    @property
    def _state_flag_names(self):
        return ()

    ### PUBLIC PROPERTIES ###

    @property
    def depth(self):
        r'''The depth of a node in a rhythm-tree structure.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()
            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.depth
            0

        ::

            >>> a[0].depth
            1

        ::

            >>> a[0][0].depth
            2

        Returns int.
        '''
        node = self
        depth = 0
        while node.parent is not None:
            depth += 1
            node = node.parent
        return depth

    @property
    def depthwise_inventory(self):
        r'''A dictionary of all nodes in a rhythm-tree, organized by their
        depth relative the root node.

        ::

            >>> a = datastructuretools.TreeContainer(name='a')
            >>> b = datastructuretools.TreeContainer(name='b')
            >>> c = datastructuretools.TreeContainer(name='c')
            >>> d = datastructuretools.TreeContainer(name='d')
            >>> e = datastructuretools.TreeContainer(name='e')
            >>> f = datastructuretools.TreeContainer(name='f')
            >>> g = datastructuretools.TreeContainer(name='g')

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])
            >>> c.extend([f, g])

        ::

            >>> inventory = a.depthwise_inventory
            >>> for depth in sorted(inventory):
            ...     print 'DEPTH: {}'.format(depth)
            ...     for node in inventory[depth]:
            ...         print node.name
            ...
            DEPTH: 0
            a
            DEPTH: 1
            b
            c
            DEPTH: 2
            d
            e
            f
            g

        Returns dictionary.
        '''
        inventory = {}
        def recurse(node):
            if node.depth not in inventory:
                inventory[node.depth] = []
            inventory[node.depth].append(node)
            if hasattr(node, 'children'):
                for child in node.children:
                    recurse(child)
        recurse(self)
        return inventory

    @property
    def graph_order(self):
        r'''Graph order of tree node.
        
        Returns tuple.
        '''
        from abjad.tools import sequencetools
        order = []
        for parent, child in sequencetools.iterate_sequence_pairwise_strict(
            reversed(self.improper_parentage)):
            order.append(parent.index(child))
        return tuple(order)

    @property
    def improper_parentage(self):
        r'''The improper parentage of a node in a rhythm-tree, being the
        sequence of node beginning with itself and ending with the root node
        of the tree.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.improper_parentage == (a,)
            True

        ::

            >>> b.improper_parentage == (b, a)
            True

        ::

            >>> c.improper_parentage == (c, b, a)
            True

        Returns tuple of tree nodes.
        '''
        node = self
        parentage = [node]
        while node.parent is not None:
            node = node.parent
            parentage.append(node)
        return tuple(parentage)

    @property
    def parent(self):
        r'''Parent of tree node.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.parent is None
            True

        ::

            >>> b.parent is a
            True

        ::

            >>> c.parent is b
            True

        Returns tree node.
        '''
        return self._parent

    @property
    def proper_parentage(self):
        r'''The proper parentage of a node in a rhythm-tree, being the
        sequence of node beginning with the node's immediate parent and
        ending with the root node of the tree.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.proper_parentage == ()
            True

        ::

            >>> b.proper_parentage == (a,)
            True

        ::

            >>> c.proper_parentage == (b, a)
            True

        Returns tuple of tree nodes.
        '''
        return self.improper_parentage[1:]

    @property
    def root(self):
        r'''The root node of the tree: that node in the tree which has
        no parent.

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.root is a
            True
            >>> b.root is a
            True
            >>> c.root is a
            True

        Returns tree node.
        '''
        node = self
        while node.parent is not None:
            node = node.parent
        return node

    ### PUBLIC PROPERTIES ###

    @property
    def name(self):
        r'''Named of tree node.
        
        Returns string.
        '''
        return self._name

    @name.setter
    def name(self, arg):
        assert isinstance(arg, (str, type(None)))
        old_name = self._name
        for parent in self.proper_parentage:
            named_children = parent._named_children
            if old_name is not None:
                named_children[old_name].remove(self)
                if not named_children[old_name]:
                    del named_children[old_name]
            if arg is not None:
                if arg not in named_children:
                    named_children[arg] = set([self])
                else:
                    named_children[arg].add(self)
        self._name = arg
